@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.h,v 1.59 2004/04/26 21:31:44 f1rmb Exp $
+ * $Id: actions.h,v 1.62 2005/07/16 19:05:30 miguelfreitas Exp $
  *
  * all the various actions for the gui (play, stop, open, pause...)
  *
@@ -38,6 +38,7 @@ void raise_window(Window window, int visible, int running);
 void toggle_window(Window window, xitk_widget_list_t *widget_list, int *visible, int running);
 
 int gui_xine_get_pos_length(xine_stream_t *stream, int *pos, int *time, int *length);
+void wait_for_window_visible(Display *display, Window window);
 void try_to_set_input_focus(Window window);
 void gui_display_logo(void);
 int gui_xine_play(xine_stream_t *stream, int start_pos, int start_time, int update_mmk);
@@ -50,6 +51,7 @@ void gui_stop (xitk_widget_t *w, void *data);
 void gui_pause (xitk_widget_t *w, void *data, int state) ;
 void gui_eject(xitk_widget_t *w, void *data);
 void gui_toggle_visibility(xitk_widget_t *w, void *data);
+void gui_toggle_border(xitk_widget_t *w, void *data);
 void gui_set_fullscreen_mode(xitk_widget_t *w, void *data);
 #ifdef HAVE_XINERAMA
 void gui_set_xinerama_fullscreen_mode(void);
@@ -86,6 +88,8 @@ void change_audio_vol(int value);
 void layer_above_video(Window w);
 void gui_increase_audio_volume(void);
 void gui_decrease_audio_volume(void);
+void gui_app_show(xitk_widget_t *w, void *data);
+void gui_app_enable(void);
 void change_amp_vol(int value);
 void gui_increase_amp_level(void);
 void gui_decrease_amp_level(void);
