@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2007 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: file_browser.c,v 1.59 2005/05/29 10:43:40 valtri Exp $
+ * $Id: file_browser.c,v 1.68 2007/04/01 22:08:35 fsck-p Exp $
  *
  */
 
@@ -43,7 +43,8 @@
 #include "common.h"
 
 #define WINDOW_WIDTH        500
-#define WINDOW_HEIGHT       391
+#define WINDOW_HEIGHT       390
+#define MAX_DISP_ENTRIES    10
 
 #define MAXFILES            65535
 
@@ -81,40 +82,40 @@ typedef struct {
 
 static filebrowser_filter_t __fb_filters[] = {
   { NULL, "*"                                                                      },
- /* subtitles */
-  { NULL, ".sub .srt .asc .smi .ssa .txt"                                          },
+  /* subtitles */
+  { NULL, ".sub .srt .asc .smi .ssa .txt "                                         },
   /* playlists */
   { NULL, ".pls .m3u .sfv .tox .asx .smil .smi .xml .fxd "                         },
   { "*.4xm", ".4xm "                                                               },
   { "*.ac3", ".ac3 "                                                               },
-  { "*.aif, *.aiff,", ".aif .aiff "                                                },
+  { "*.aif, *.aiff", ".aif .aiff "                                                 },
   {"*.asf, *.wmv, *.wma, *.wvx, *.wax", ".asf .wmv .wma .wvx .wax "                },
   { "*.aud", ".aud "                                                               },
   { "*.avi", ".avi "                                                               },
-  { "*.cin,", ".cin "                                                              },
-  { "*.cpk, *.cak, *.film,", ".cpk .cak .film "                                    },
+  { "*.cin", ".cin "                                                               },
+  { "*.cpk, *.cak, *.film", ".cpk .cak .film "                                     },
   { "*.dv, *.dif", ".dv .dif "                                                     },
-  { "*.fli, *.flc,", ".fli .flc "                                                  },
+  { "*.fli, *.flc", ".fli .flc "                                                   },
   { "*.mp3, *.mp2, *.mpa, *.mpega", ".mp3 .mp2 .mpa .mpega "                       },
   { "*.mjpg", ".mjpg "                                                             },
-  { "*.mov, *.qt, *.mp4,", ".mov .qt .mp4 "                                        },
+  { "*.mov, *.qt, *.mp4", ".mov .qt .mp4 "                                         },
   { "*.mpg, *.mpeg", ".mpg .mpeg "                                                 },
   { "*.mpv", ".mpv "                                                               },
   { "*.mve, *.mv8", ".mve .mv8 "                                                   },
   { "*.nsf", ".nsf "                                                               },
   { "*.nsv", ".nsv "                                                               },
-  { "*.ogg, *.ogm, *.spx,", ".ogg .ogm .spx "                                      },
+  { "*.ogg, *.ogm, *.spx", ".ogg .ogm .spx "                                       },
   { "*.pes", ".pes "                                                               },
-  { "*.png, *.mng,", ".png .mng "                                                  },
+  { "*.png, *.mng", ".png .mng "                                                   },
   { "*.pva", ".pva "                                                               },
   { "*.ra", ".ra "                                                                 },
   { "*.rm, *.ram", ".rm .ram "                                                     },
   { "*.roq", ".roq "                                                               },
-  { "*.str, *.iki, *.ik2, *.dps, *.dat, *.xa1, *.xa2, *.xas, *.xap, *.xa,",
+  { "*.str, *.iki, *.ik2, *.dps, *.dat, *.xa1, *.xa2, *.xas, *.xap, *.xa",
     ".str .iki .ik2 .dps .dat .xa1 .xa2 .xas .xap .xa "                            },
   { "*.snd, *.au", ".snd .au "                                                     },
-  { "*.ts, *.m2t, *.trp,", ".ts .m2t .trp "                                        },
-  { "*.vqa,", ".vqa "                                                              },
+  { "*.ts, *.m2t, *.trp", ".ts .m2t .trp "                                         },
+  { "*.vqa", ".vqa "                                                               },
   { "*.vob", ".vob "                                                               },
   { "*.voc", ".voc "                                                               },
   { "*.vox", ".vox "                                                               },
@@ -289,6 +290,8 @@ static void fne_handle_event(XEvent *event, void *data) {
   case KeyPress:
     if(xitk_get_key_pressed(event) == XK_Escape)
       fne_cancel_cb(NULL, data);
+    else
+      gui_handle_event(event, data);
     break;
   }
 }
@@ -298,7 +301,7 @@ static void fb_create_input_window(char *title, char *text,
   GC                          gc;
   int                         x, y, w;
   int                         width = WINDOW_WIDTH;
-  int                         height = 100;
+  int                         height = 102;
   xitk_labelbutton_widget_t   lb;
   xitk_inputtext_widget_t     inp;
   
@@ -333,7 +336,7 @@ static void fb_create_input_window(char *title, char *text,
   XITK_WIDGET_INIT(&inp, gGui->imlib_data);
 
   x = 15;
-  y = 23;
+  y = 30;
   w = width - 30;
   
   inp.skin_element_name = NULL;
@@ -347,8 +350,8 @@ static void fb_create_input_window(char *title, char *text,
 					  x, y, w, 20, "Black", "Black", fontname)));
   xitk_enable_and_show_widget(fne->input);
 
-  y = height - 23 - 15;
-  x = width - (100 * 2) - (15 * 2);
+  y = height - (23 + 15);
+  x = 15;
   w = 100;
 
   lb.button_type       = CLICK_BUTTON;
@@ -365,7 +368,7 @@ static void fb_create_input_window(char *title, char *text,
 					   "Black", "Black", "White", btnfontname)));
   xitk_enable_and_show_widget(fne->button_apply);
 
-  x += 100 + 15;
+  x = width - (w + 15);
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Cancel");
@@ -444,62 +447,74 @@ static int is_file_match_to_filter(filebrowser_t *fb, char *file) {
 }
 
 static void fb_update_origin(filebrowser_t *fb) {
-  char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+  char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
   
-  if(strcasecmp(fb->current_dir, "/")) {
-    if(strlen(fb->filename))
-      snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->filename);
-    else
-      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
-  }
-  else {
-    if(strlen(fb->filename))
-      snprintf(buf, sizeof(buf), "%s%s", fb->current_dir, fb->filename);
-    else
-      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
-  }
+  if(strcmp(fb->current_dir, "/"))
+    snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->filename);
+  else
+    snprintf(buf, sizeof(buf), "/%s", fb->filename);
   
   xitk_inputtext_change_text(fb->origin, buf); 
 }
 
 static void fb_extract_path_and_file(filebrowser_t *fb, char *filepathname) {
-  if(filepathname) {
+  if(filepathname && *filepathname) {
+    char *dirname = NULL;
     char *filename = NULL;
-    char *_filepathname;
-    char *tilde;
+    char  _filepathname[XITK_PATH_MAX + XITK_NAME_MAX + 2];
+    char *p;
     
-    if((tilde = strchr(filepathname, '~')) && (*(tilde - 1) != '\\')) {
+    if((*filepathname == '~') && (*(filepathname + 1) == '/' || *(filepathname + 1) == '\0')) {
       const char *homedir = xine_get_homedir();
-      int         len = strlen(homedir) + (tilde - filepathname);
       
-      tilde++;
-      
-      if(*tilde == '/')
-	len += strlen(tilde);
-      
-      _filepathname = (char *) alloca(len + 1);
-      sprintf(_filepathname, "%s%s", homedir, (tilde) ? tilde : "");
-    }
-    else 
-      xine_strdupa(_filepathname, filepathname);
-
-    if(!is_a_dir((char *)_filepathname))
-      filename = strrchr(_filepathname, '/');
-    
-    if(filename && (strlen(filename) > 1)) {
-      *filename++ = '\0';
-      snprintf(fb->filename, sizeof(fb->filename), "%s", filename);
-      
-      if(is_a_dir(_filepathname))
-	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", _filepathname);
-
+      snprintf(_filepathname, sizeof(_filepathname), "%s%s", homedir, (filepathname + 1));
     }
     else {
-      if(is_a_dir((char *)_filepathname)) {
-	snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", _filepathname);
-	memset(&fb->filename, 0, sizeof(fb->filename));
+      if((*filepathname == '\\') && (*(filepathname + 1) == '~'))
+	filepathname++;
+      if((*filepathname == '/'))
+	snprintf(_filepathname, sizeof(_filepathname), "%s", filepathname);
+      else
+	snprintf(_filepathname, sizeof(_filepathname), "%s/%s", fb->current_dir, filepathname);
+    }
+
+    p = _filepathname + strlen(_filepathname) - 1;
+    while((p > _filepathname) && (*p == '/'))	/* Remove trailing '/' from path name */
+      *p-- = '\0';
+
+    if(is_a_dir(_filepathname)) {		/* Whole name is a dir name */
+      dirname = _filepathname;
+    }
+    else {
+      filename = strrchr(_filepathname, '/');
+    
+      if(!filename) {				/* Whole name treated as file name */
+	filename = _filepathname;
+      }
+      else {					/* Split into dir and file part */
+	*filename++ = '\0';
+
+	if(*_filepathname == '\0')
+	  dirname = "/";			/* Dir part was "/", restore it */
+	else
+	  dirname = _filepathname;
+
+	p = dirname + strlen(dirname) - 1;
+	while((p > dirname) && (*p == '/'))	/* Remove trailing '/' from dir name */
+	  *p-- = '\0';
+
+	if(!is_a_dir(dirname))			/* Invalid path, don't change anything */
+	  return;
       }
     }
+
+    if(dirname)
+      snprintf(fb->current_dir, sizeof(fb->current_dir), "%s", dirname);
+
+    if(filename)
+      snprintf(fb->filename, sizeof(fb->filename), "%s", filename);
+    else
+      *fb->filename = '\0';
   }
 }
 
@@ -513,7 +528,7 @@ static void fb_extract_path_and_file(filebrowser_t *fb, char *filepathname) {
 #define CMP          2
 #define LEN          3
 #define ISDIGIT(c)   ((unsigned) (c) - '0' <= 9)
-static int strverscmp(const char *s1, const char *s2) {
+static int my_strverscmp(const char *s1, const char *s2) {
   const unsigned char *p1 = (const unsigned char *) s1;
   const unsigned char *p2 = (const unsigned char *) s2;
   unsigned char c1, c2;
@@ -570,13 +585,13 @@ static int strverscmp(const char *s1, const char *s2) {
 }
 
 /*
- * Wrapper to strverscmp() for qsort() calls, which sort mrl_t type array.
+ * Wrapper to my_strverscmp() for qsort() calls, which sort mrl_t type array.
  */
 static int _sortfiles_default(const fileinfo_t *s1, const fileinfo_t *s2) {
-  return(strverscmp(s1->name, s2->name));
+  return(my_strverscmp(s1->name, s2->name));
 }
 static int _sortfiles_reverse(const fileinfo_t *s1, const fileinfo_t *s2) {
-  return(strverscmp(s2->name, s1->name));
+  return(my_strverscmp(s2->name, s1->name));
 }
 
 static void sort_files(filebrowser_t *fb) {
@@ -648,7 +663,7 @@ static void sort_directories(filebrowser_t *fb) {
 }
 
 static void fb_getdir(filebrowser_t *fb) {
-  char                  fullfilename[XINE_PATH_MAX + XINE_NAME_MAX + 1];
+  char                  fullfilename[XINE_PATH_MAX + XINE_NAME_MAX + 2];
   struct dirent        *pdirent;
   int                   num_dir_files   = 0;
   int                   num_norm_files  = 0;
@@ -671,24 +686,28 @@ static void fb_getdir(filebrowser_t *fb) {
     }
   }
   
+  /* Following code relies on the fact that fb->current_dir has no trailing '/' */
   if((pdir = opendir(fb->current_dir)) == NULL) {
     char *p = strrchr(fb->current_dir, '/');
     
     xine_error(_("Unable to open directory '%s': %s."), 
-	       (p) ? p + 1 : fb->current_dir, strerror(errno));
+	       (p && *(p + 1)) ? p + 1 : fb->current_dir, strerror(errno));
     
-    /* One step back */
-    if(p)
-      *p = '\0';
+    /* One step back if dir has a subdir component */
+    if(p && *(p + 1)) {
+      if(p == fb->current_dir) /* we are in the root dir */
+	*(p + 1) = '\0';
+      else
+	*p = '\0';
 
-    fb_update_origin(fb);
-    fb_getdir(fb);
+      fb_update_origin(fb);
+      fb_getdir(fb);
+    }
     return;
   }
 
   while((pdirent = readdir(pdir)) != NULL) {
     
-    memset(&fullfilename, 0, sizeof(fullfilename));
     snprintf(fullfilename, sizeof(fullfilename), "%s/%s", fb->current_dir, pdirent->d_name);
     
     if(is_a_dir(fullfilename)) {
@@ -777,7 +796,7 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
   filebrowser_t *fb = (filebrowser_t *) data;
 
   if(w == fb->directories_browser) {
-    char buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+    char buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
     
     /* Want to re-read current dir */
     if(!strcasecmp(fb->dir_files[selected].name, ".")) {
@@ -786,7 +805,6 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
     else if(!strcasecmp(fb->dir_files[selected].name, "..")) {
       char *p;
       
-      memset(&buf, '\0', sizeof(buf));
       snprintf(buf, sizeof(buf), "%s", fb->current_dir);
       if(strlen(buf) > 1) { /* not '/' directory */
 	
@@ -807,11 +825,9 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
       
       /* not '/' directory */
       if(strcasecmp(fb->current_dir, "/")) {
-	memset(&buf, 0, sizeof(buf));
 	snprintf(buf, sizeof(buf), "%s/%s", fb->current_dir, fb->dir_files[selected].name);
       }
       else {
-	memset(&buf, 0, sizeof(buf));
 	snprintf(buf, sizeof(buf), "/%s", fb->dir_files[selected].name);
       }
       
@@ -832,17 +848,8 @@ static void fb_dbl_select(xitk_widget_t *w, void *data, int selected) {
 
 static void fb_change_origin(xitk_widget_t *w, void *data, char *currenttext) {
   filebrowser_t *fb = (filebrowser_t *)data;
-  char           buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  char          *p;
 
-  memset(&buf, 0, sizeof(buf));
-  snprintf(buf, sizeof(buf), "%s", currenttext);
-
-  p = &buf[strlen(buf) - 1];
-  while((strlen(buf) > 1) && (*p == '/'))
-    *p-- = '\0';
-
-  fb_extract_path_and_file(fb, buf);
+  fb_extract_path_and_file(fb, currenttext);
   fb_update_origin(fb);
   fb_getdir(fb);
 }
@@ -976,14 +983,12 @@ static void fb_delete_file_cb(xitk_widget_t *w, void *data, int button) {
   switch(button) {
   case XITK_WINDOW_ANSWER_YES:
     {
-      char buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+      char buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
       int sel = xitk_browser_get_current_selected(fb->files_browser);
 
-      memset(&buf, 0, sizeof(buf));
-      snprintf(buf, sizeof(buf), "%s", fb->current_dir);
-      if(strlen(fb->current_dir) > 1)
-	strcat(buf, "/");
-      sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
+      snprintf(buf, sizeof(buf), "%s%s%s",
+	       fb->current_dir, ((fb->current_dir[0] && strcmp(fb->current_dir, "/")) ? "/" : ""),
+	       fb->norm_files[sel].name);
       
       if((unlink(buf)) == -1)
 	xine_error(_("Unable to delete file '%s': %s."), buf, strerror(errno));
@@ -1001,32 +1006,29 @@ static void fb_delete_file(xitk_widget_t *w, void *data) {
   int            sel;
   
   if((sel = xitk_browser_get_current_selected(fb->files_browser)) >= 0) {
-    char buf[256 + XITK_PATH_MAX + XITK_NAME_MAX + 1];
+    char buf[256 + XITK_PATH_MAX + XITK_NAME_MAX + 2];
 
-    snprintf(buf, sizeof(buf), _("Do you really want to delete the file: '%s'"), fb->current_dir);
-    if(strlen(fb->current_dir) > 1)
-      strcat(buf, "/");
-    sprintf(buf, "%s%s' ?.", buf, fb->norm_files[sel].name);
+    snprintf(buf, sizeof(buf), _("Do you really want to delete the file '%s%s%s' ?"),
+	     fb->current_dir, ((fb->current_dir[0] && strcmp(fb->current_dir, "/")) ? "/" : ""),
+	     fb->norm_files[sel].name);
     
     fb_deactivate(fb);
     xitk_window_dialog_yesno(gGui->imlib_data, _("Confirm deletion ?"),
 			     fb_delete_file_cb, 
 			     fb_delete_file_cb, 
-			     (void *)fb, ALIGN_DEFAULT, buf);
+			     (void *)fb, ALIGN_DEFAULT, "%s", buf);
   }
 }
 
 static void fb_rename_file_cb(xitk_widget_t *w, void *data, char *newname) {
   filebrowser_t *fb = (filebrowser_t *) data;
-  char buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+  char buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
   int sel = xitk_browser_get_current_selected(fb->files_browser);
   
-  memset(&buf, 0, sizeof(buf));
-  snprintf(buf, sizeof(buf), "%s", fb->current_dir);
-  if(strlen(fb->current_dir) > 1)
-    strcat(buf, "/");
-  sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
-  
+  snprintf(buf, sizeof(buf), "%s%s%s",
+	   fb->current_dir, ((fb->current_dir[0] && strcmp(fb->current_dir, "/")) ? "/" : ""),
+	   fb->norm_files[sel].name);
+      
   if((rename(buf, newname)) == -1)
     xine_error(_("Unable to rename file '%s' to '%s': %s."), buf, newname, strerror(errno));
   else
@@ -1038,14 +1040,12 @@ static void fb_rename_file(xitk_widget_t *w, void *data) {
   int            sel;
   
   if((sel = xitk_browser_get_current_selected(fb->files_browser)) >= 0) {
-    char buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-    
-    memset(&buf, 0, sizeof(buf));
-    snprintf(buf, sizeof(buf), "%s", fb->current_dir);
-    if(strlen(fb->current_dir) > 1)
-      strcat(buf, "/");
-    sprintf(buf, "%s%s", buf, fb->norm_files[sel].name);
+    char buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
     
+    snprintf(buf, sizeof(buf), "%s%s%s",
+	     fb->current_dir, ((fb->current_dir[0] && strcmp(fb->current_dir, "/")) ? "/" : ""),
+	     fb->norm_files[sel].name);
+      
     fb_deactivate(fb);
     fb_create_input_window(_("Rename file"), buf, fb_rename_file_cb, fb);
   }
@@ -1061,13 +1061,11 @@ static void fb_create_directory_cb(xitk_widget_t *w, void *data, char *newdir) {
 }
 static void fb_create_directory(xitk_widget_t *w, void *data) {
   filebrowser_t *fb = (filebrowser_t *) data;
-  char           buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  
-  memset(&buf, 0, sizeof(buf));
-  snprintf(buf, sizeof(buf), "%s", fb->current_dir);
-  if(strlen(fb->current_dir) > 1)
-    strcat(buf, "/");
+  char           buf[XITK_PATH_MAX + XITK_NAME_MAX + 2];
   
+  snprintf(buf, sizeof(buf), "%s%s",
+	   fb->current_dir, ((fb->current_dir[0] && strcmp(fb->current_dir, "/")) ? "/" : ""));
+      
   fb_deactivate(fb);
   fb_create_input_window(_("Create a new directory"), buf, fb_create_directory_cb, fb);
 }
@@ -1099,13 +1097,17 @@ static void fb_handle_events(XEvent *event, void *data) {
   switch(event->type) {
 
   case KeyPress:
-    if(xitk_get_key_pressed(event) == XK_Escape)
-      _fb_exit(NULL, data);
+    if(xitk_get_key_pressed(event) == XK_Escape) {
+      if(xitk_is_widget_enabled(fb->close)) /* Exit only if close button would exit */
+	_fb_exit(NULL, data);
+    }
     else {
       int sel = xitk_browser_get_current_selected(fb->files_browser);
       
       if(sel >= 0)
 	fb_select(fb->files_browser, (void *) fb, sel);
+      else
+	gui_handle_event(event, data);
     }
     break;
   }
@@ -1296,14 +1298,14 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 
   fb_update_origin(fb);
 
-  y += 46;
-  w = (WINDOW_WIDTH >> 1) - 15 - 12 - 10;
+  y += 45;
+  w = (WINDOW_WIDTH - 30 - 10) / 2;
 
   br.arrow_up.skin_element_name    = NULL;
   br.slider.skin_element_name      = NULL;
   br.arrow_dn.skin_element_name    = NULL;
   br.browser.skin_element_name     = NULL;
-  br.browser.max_displayed_entries = 10;
+  br.browser.max_displayed_entries = MAX_DISP_ENTRIES;
   br.browser.num_entries           = fb->directories_num;
   br.browser.entries               = (const char *const *)fb->directories;
   br.callback                      = fb_select;
@@ -1314,31 +1316,30 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 	   (fb->directories_browser = 
 	    xitk_noskin_browser_create(fb->widget_list, &br,
 				       (XITK_WIDGET_LIST_GC(fb->widget_list)),
-				       x, y, w, 20, 12, fontname)));
+				       x + 2, y + 2, w - 4 - 12, 20, 12, fontname)));
   xitk_enable_and_show_widget(fb->directories_browser);
   
-  draw_rectangular_inner_box(gGui->imlib_data, bg, x - 1, y - 1,
-			     xitk_get_widget_width(fb->directories_browser) + 2,
-			     xitk_get_widget_height(fb->directories_browser) + 2);
+  draw_rectangular_inner_box(gGui->imlib_data, bg, x, y,
+			     w - 1, xitk_get_widget_height(fb->directories_browser) + 4 - 1);
   
-  y -= 16;
+  y -= 15;
   
   b.skin_element_name = NULL;
   b.callback          = fb_sort;
   b.userdata          = (void *)fb;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(fb->widget_list)), 
 	   (fb->directories_sort =
-	    xitk_noskin_button_create(fb->widget_list, &b, x - 1, y, w + 12 + 2, 15)));
+	    xitk_noskin_button_create(fb->widget_list, &b, x, y, w, 15)));
   xitk_enable_and_show_widget(fb->directories_sort);
 
-  x = WINDOW_WIDTH - (w + 15 + 12);
-  y += 16;
+  x = WINDOW_WIDTH - (w + 15);
+  y += 15;
 
   br.arrow_up.skin_element_name    = NULL;
   br.slider.skin_element_name      = NULL;
   br.arrow_dn.skin_element_name    = NULL;
   br.browser.skin_element_name     = NULL;
-  br.browser.max_displayed_entries = 10;
+  br.browser.max_displayed_entries = MAX_DISP_ENTRIES;
   br.browser.num_entries           = fb->files_num;
   br.browser.entries               = (const char *const *)fb->files;
   br.callback                      = fb_select;
@@ -1349,21 +1350,20 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 	   (fb->files_browser = 
 	    xitk_noskin_browser_create(fb->widget_list, &br,
 				       (XITK_WIDGET_LIST_GC(fb->widget_list)),
-				       x, y, w, 20, 12, fontname)));
+				       x + 2, y + 2, w - 4 - 12, 20, 12, fontname)));
   xitk_enable_and_show_widget(fb->files_browser);
 
-  draw_rectangular_inner_box(gGui->imlib_data, bg, x - 1, y - 1,
-			     xitk_get_widget_width(fb->files_browser) + 2,
-			     xitk_get_widget_height(fb->files_browser) + 2);
+  draw_rectangular_inner_box(gGui->imlib_data, bg, x, y,
+			     w - 1, xitk_get_widget_height(fb->files_browser) + 4 - 1);
 
-  y -= 16;
+  y -= 15;
 
   b.skin_element_name = NULL;
   b.callback          = fb_sort;
   b.userdata          = (void *)fb;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(fb->widget_list)), 
 	   (fb->files_sort =
-	    xitk_noskin_button_create(fb->widget_list, &b, x - 1, y, w + 12 + 2, 15)));
+	    xitk_noskin_button_create(fb->widget_list, &b, x, y, w, 15)));
   xitk_enable_and_show_widget(fb->files_sort);
 
 
@@ -1455,7 +1455,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 	XSetForeground(gGui->display, image->image->gc, 
 		       xitk_get_pixel_color_lightgray(gGui->imlib_data));
 	XFillPolygon(gGui->display, image->image->pixmap, image->image->gc, 
-		     &points[0], 4, Complex, CoordModeOrigin);
+		     &points[0], 4, Convex, CoordModeOrigin);
 	XUnlockDisplay(gGui->display);
 	
 	XLockDisplay(gGui->display);
@@ -1484,9 +1484,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 			    fsimage, fsimage->width, fsimage->height);
   }
 
-  x--;
-  y += xitk_get_widget_height(fb->files_browser) + 16 + 5;
-  w = xitk_get_widget_width(fb->files_browser);
+  y += xitk_get_widget_height(fb->files_browser) + 15 + 4 + 5;
 
   cmb.skin_element_name = NULL;
   cmb.layer_above       = (is_layer_above());
@@ -1523,8 +1521,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 	    xitk_noskin_label_create(fb->widget_list, &lbl, x + 15, y, w - 15, 20, fontname)));
   xitk_enable_and_show_widget(widget);
   
-
-  y += 30;
+  y = WINDOW_HEIGHT - (23 + 15) - (23 + 8);
   w = (WINDOW_WIDTH - (4 * 15)) / 3;
 
   lb.button_type       = CLICK_BUTTON;
@@ -1541,7 +1538,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 					   "Black", "Black", "White", btnfontname)));
   xitk_enable_and_show_widget(fb->rename);
 
-  x += w + 15;
+  x = (WINDOW_WIDTH - w) / 2;
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Delete");
@@ -1557,7 +1554,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 					   "Black", "Black", "White", btnfontname)));
   xitk_enable_and_show_widget(fb->delete);
   
-  x += w + 15;
+  x = WINDOW_WIDTH - (w + 15);
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Create a directory");
@@ -1575,9 +1572,10 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
 
   fb->cb_buttons[0] = fb->cb_buttons[1] = NULL;
 
+  y = WINDOW_HEIGHT - (23 + 15);
+
   if(fb->cbb[0].label) {
     x = 15;
-    y += 30;
     
     lb.button_type       = CLICK_BUTTON;
     lb.label             = fb->cbb[0].label;
@@ -1594,7 +1592,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
     xitk_enable_and_show_widget(fb->cb_buttons[0]);
 
     if(fb->cbb[1].label) {
-      x += w + 15;
+      x = (WINDOW_WIDTH - w) / 2;
       
       lb.button_type       = CLICK_BUTTON;
       lb.label             = fb->cbb[1].label;
@@ -1612,8 +1610,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname, hidden
     }
   }
 
-  x = WINDOW_WIDTH - w - 15;
-  y = WINDOW_HEIGHT - 23 - 15;
+  x = WINDOW_WIDTH - (w + 15);
   
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
