@@ -1,6 +1,6 @@
 #warning IMPLEMENT POST SUPPORT
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -18,14 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: network.c,v 1.65 2005/07/16 19:05:32 miguelfreitas Exp $
+ * $Id: network.c,v 1.76 2006/07/15 20:26:20 dgp85 Exp $
  *
  * xine network related stuff
  *
  */
-/* required for getsubopt(); the __sun test avoids compilation problems on */
-/* solaris */
-#if ! defined (__sun__) && ! defined (__OpenBSD__) 
+/* required for getsubopt(); the __sun test avoids compilation problems on 
+    solaris. On FreeBSD defining this disable BSD functions to be visible
+    and remove INADDR_NONE */
+#if ! defined (__sun__) && ! defined (__OpenBSD__)  && ! defined(__FreeBSD__)
 #define _XOPEN_SOURCE 500
 #endif
 /* required for strncasecmp() */
@@ -33,9 +34,7 @@
 /* required to enable POSIX variant of getpwuid_r on solaris */
 #define _POSIX_PTHREAD_SEMANTICS 1
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #ifdef HAVE_READLINE
 
@@ -56,9 +55,10 @@
 #include <unistd.h>
 #include <time.h>
 #include <sys/ioctl.h>
+#include <sys/select.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -225,7 +225,10 @@ struct client_info_s {
   int                   finished;
 
   int                   socket;
-  struct sockaddr_in    sin;
+  union {
+    struct sockaddr_in  in;
+    struct sockaddr     sa;
+  } fsin;
   char                **actions;
 
   struct {
@@ -432,7 +435,7 @@ extern gGui_t *gGui;
 
 #endif
 
-static void sock_err(const char *error_msg, ...) {
+static void __attribute__ ((format (printf, 1, 2))) sock_err(const char *error_msg, ...) {
   va_list args;
   
   va_start(args, error_msg);
@@ -539,7 +542,7 @@ static int _sock_write(int socket, char *buf, int len) {
   return wlen;
 }
 
-static int __sock_write(int socket, int cr, char *msg, ...) {
+static int __attribute__ ((format (printf, 3, 4))) __sock_write(int socket, int cr, char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
   
@@ -550,7 +553,7 @@ static int __sock_write(int socket, int cr, char *msg, ...) {
   /* Each line sent is '\n' terminated */
   if(cr) {
     if((buf[strlen(buf)] == '\0') && (buf[strlen(buf) - 1] != '\n'))
-      sprintf(buf, "%s%c", buf, '\n');
+      strlcat(buf, "\n", sizeof(buf));
   }
   
   return _sock_write(socket, buf, strlen(buf));
@@ -592,23 +595,26 @@ static char *_atoa(char *str) {
 
 #ifdef NETWORK_CLIENT
 static int sock_client(const char *host, const char *service, const char *transport) {
-  struct sockaddr_in   sin;
+  union {
+    struct sockaddr_in in;
+    struct sockaddr sa;
+  } fsin;
   struct hostent      *ihost;
   int                  sock;
   
-  sock = sock_create(service, transport, &sin);
+  sock = sock_create(service, transport, &fsin.in);
 
-  if ((sin.sin_addr.s_addr = inet_addr(host)) == -1) {
+  if ((fsin.in.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE) {
     ihost = gethostbyname(host);
     
     if(!ihost) {
       sock_err("Unknown host: %s\n", host);
       return -1;
     }
-    memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
+    memcpy(&fsin.in.sin_addr, ihost->h_addr, ihost->h_length);
   }
   
-  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
+  if(connect(sock, &fsin.sa, sizeof(fsin.in)) < 0) {
     int err = errno;
 
     close(sock);
@@ -628,7 +634,7 @@ static int sock_client(const char *host, const char *service, const char *transp
 #define write_to_console_unlocked_nocr(session, ...) __sock_write(session->console, 0, __VA_ARGS__)
 #endif
 
-static int write_to_console(session_t *session, const char *msg, ...) {
+static int __attribute__ ((format (printf, 2, 3))) write_to_console(session_t *session, const char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
   int      err;
@@ -638,14 +644,14 @@ static int write_to_console(session_t *session, const char *msg, ...) {
   va_end(args);
   
   pthread_mutex_lock(&session->console_mutex);
-  err = write_to_console_unlocked(session, buf);
+  err = write_to_console_unlocked(session, "%s", buf);
   pthread_mutex_unlock(&session->console_mutex);
 
   return err;
 }
 
 #if 0
-static int write_to_console_nocr(session_t *session, const char *msg, ...) {
+static int __attribute__ ((format (printf, 2, 3)) write_to_console_nocr(session_t *session, const char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
   int      err;
@@ -717,8 +723,8 @@ static void session_create_commands(session_t *session) {
 static void client_noop(session_t *session, session_commands_t *command, const char *cmd) {
 }
 static void client_help(session_t *session, session_commands_t *command, const char *cmd) {
-  int i = 0, j;
-  int maxlen = 0;
+  int i = 0;
+  size_t maxlen = 0, j;
   int curpos = 0;
   char buf[_BUFSIZ];
   
@@ -746,15 +752,15 @@ static void client_help(session_t *session, session_commands_t *command, const c
   while(session_commands[i]->command != NULL) {
     if(session_commands[i]->enable) {
       if((curpos + maxlen) >= 80) {
-	sprintf(buf, "%s\n       ", buf);
+	strlcat(buf, "\n       ", sizeof(buf));
 	curpos = 7;
       }
       
-      sprintf(buf, "%s%s", buf, session_commands[i]->command);
+      strlcat(buf, session_commands[i]->command, sizeof(buf));
       curpos += strlen(session_commands[i]->command);
       
       for(j = 0; j < (maxlen - strlen(session_commands[i]->command)); j++) {
-	sprintf(buf, "%s ", buf);
+	strlcat(buf, " ", sizeof(buf));
 	curpos++;
       }
     }
@@ -998,7 +1004,7 @@ static void *select_thread(void *data) {
 		  write_to_console_unlocked_nocr(session, "\b \b");
 		  pos--;
 		}
-		write_to_console_unlocked(session, obuffer);
+		write_to_console_unlocked(session, "%s", obuffer);
 
 		rl_crlf();
 		rl_forced_update_display();
@@ -1082,7 +1088,7 @@ static void client_handle_command(session_t *session, const char *command) {
 	  
 	  *pp = '\0';
 	  
-	  if((sock_write(session->socket, buf)) == -1) {
+	  if((sock_write(session->socket, "%s", buf)) == -1) {
 	    session->running = 0;
 	  }
 	}
@@ -1094,7 +1100,7 @@ static void client_handle_command(session_t *session, const char *command) {
   
   /* Perhaps a ';' separated commands, so send anyway to server */
   if(found == 0) {
-    sock_write(session->socket, (char *)command);
+    sock_write(session->socket, "%s", (char *)command);
   }
   
   if((!strncasecmp(cmd, "exit", strlen(cmd))) || (!strncasecmp(cmd, "halt", strlen(cmd)))) {
@@ -1108,13 +1114,13 @@ static void session_single_shot(session_t *session, int num_commands, char *comm
   int i;
   char buf[_BUFSIZ];
   
-  memset(&buf, 0, sizeof(buf));
+  buf[0] = 0;
 
   for(i = 0; i < num_commands; i++) {
-    if(strlen(buf))
-      sprintf(buf, "%s %s", buf, commands[i]);
+    if(buf[0])
+      sprintf(buf+strlen(buf), " %s", commands[i]);
     else
-      snprintf(buf, sizeof(buf), "%s", commands[i]);
+      strcpy(buf, commands[i]);
   }
 
   client_handle_command(session, buf);
@@ -1326,15 +1332,18 @@ int main(int argc, char **argv) {
 #else
 
 static int sock_serv(const char *service, const char *transport, int queue_length) {
-  struct sockaddr_in  sin;
+  union {
+    struct sockaddr_in in;
+    struct sockaddr sa;
+  } fsin;
   int                 sock;
   int                 on = 1;
 
-  sock = sock_create(service, transport, &sin);
+  sock = sock_create(service, transport, &fsin.in);
   
   setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
   
-  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
+  if(bind(sock, &fsin.sa, sizeof(fsin.in)) < 0)
     sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
   
   if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
@@ -1662,8 +1671,7 @@ static void set_command_line(client_info_t *client_info, char *line) {
 
     client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
     
-    memset(client_info->command.line, 0, sizeof(client_info->command.line));
-    sprintf(client_info->command.line, "%s", line);
+    strcpy(client_info->command.line, line);
 
   }
 }
@@ -1704,25 +1712,24 @@ static void do_commands(commands_t *cmd, client_info_t *client_info) {
   int i = 0;
   char buf[_BUFSIZ];
 
-  memset(&buf, 0, sizeof(buf));
-  snprintf(buf, sizeof(buf), "%s", COMMANDS_PREFIX);
+  strcpy(buf, COMMANDS_PREFIX);
   
   while(commands[i].command != NULL) {
     if(commands[i].public) {
-      sprintf(buf, "%s\t%s", buf, commands[i].command);
+      snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "\t%s", commands[i].command);
     }
     i++;
   }
-  sprintf(buf, "%s.\n", buf);
-  sock_write(client_info->socket, buf);
+  strlcat(buf, ".\n", sizeof(buf));
+  sock_write(client_info->socket, "%s", buf);
 }
 
 static void do_help(commands_t *cmd, client_info_t *client_info) {
   char buf[_BUFSIZ];
 
   if(!client_info->command.num_args) {
-    int i = 0, j;
-    int maxlen = 0;
+    int i = 0;
+    size_t maxlen = 0, j;
     int curpos = 0;
     
     while(commands[i].command != NULL) {
@@ -1744,23 +1751,23 @@ static void do_help(commands_t *cmd, client_info_t *client_info) {
     while(commands[i].command != NULL) {
       if(commands[i].public) {
 	if((curpos + maxlen) >= 80) {
-	  sprintf(buf, "%s\n       ", buf);
+	  strlcat(buf, "\n       ", sizeof(buf));
 	  curpos = 7;
 	}
 	
-	sprintf(buf, "%s%s", buf, commands[i].command);
+	strlcat(buf, commands[i].command, sizeof(buf));
 	curpos += strlen(commands[i].command);
 	
 	for(j = 0; j < (maxlen - strlen(commands[i].command)); j++) {
-	  sprintf(buf, "%s ", buf);
+	  strlcat(buf, " ", sizeof(buf));
 	  curpos++;
 	}
       }
       i++;
     }
     
-    sprintf(buf, "%s\n", buf);
-    sock_write(client_info->socket, buf);
+    strlcat(buf, "\n", sizeof(buf));
+    sock_write(client_info->socket, "%s", buf);
   }
   else {
     int i;
@@ -1919,12 +1926,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
       else if((first = is_arg_contain(client_info, 1, "first")) || is_arg_contain(client_info, 1, "last")) {
 
 	if(gGui->playlist.num) {
-	  int entry = gGui->playlist.cur;
-	  
-	  if(entry)
-	    entry = 0;
-	  else
-	    entry = gGui->playlist.num - 1;
+	  int entry = (first) ? 0 : gGui->playlist.num - 1;
 	  
 	  if(entry != gGui->playlist.cur) {
 	    
@@ -1946,7 +1948,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
       }
       else if(is_arg_contain(client_info, 1, "continue")) {
 	if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
-	  gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+	  gGui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
       }
 
     }
@@ -2087,22 +2089,22 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	char buf[64];
 	int  status;
 
-	snprintf(buf, sizeof(buf), "%s", "Current status: ");
+	strcpy(buf, "Current status: ");
 	status = xine_get_status(gGui->stream);
 	
 	if(status <= XINE_STATUS_QUIT)
-	  sprintf(buf, "%s%s", buf, status_struct[status].name);
+	  strlcat(buf, status_struct[status].name, sizeof(buf));
 	else
-	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
+	  strlcat(buf, "*UNKNOWN*", sizeof(buf));
 	
-	sprintf(buf, "%s%c", buf, '\n');
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s\n", buf);
       }
       else if(is_arg_contain(client_info, 1, "speed")) {
 	char buf[64];
-	int  speed = -1, i;
+	int  speed = -1;
+	size_t i;
 	
-	snprintf(buf, sizeof(buf), "%s", "Current speed: ");
+	strcpy(buf, "Current speed: ");
 	speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
 
 	for(i = 0; speeds_struct[i].name != NULL; i++) {
@@ -2111,12 +2113,11 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	}
 	
 	if(i < ((sizeof(speeds_struct) / sizeof(speeds_struct[0])) - 1))
-	  sprintf(buf, "%s%s", buf, speeds_struct[i].name);
+	  strlcat(buf, speeds_struct[i].name, sizeof(buf));
 	else
-	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
+	  strlcat(buf, "*UNKNOWN*", sizeof(buf));
 	
-	sprintf(buf, "%s%c", buf, '\n');
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s\n", buf);
       }
       else if(is_arg_contain(client_info, 1, "position")) {
 	char buf[64];
@@ -2128,7 +2129,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_time,
 			    &length_time);
 	snprintf(buf, sizeof(buf), "%s: %d\n", "Current position", pos_time);
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s", buf);
       }
       else if(is_arg_contain(client_info, 1, "length")) {
 	char buf[64];
@@ -2140,7 +2141,7 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_time,
 			    &length_time);
 	snprintf(buf, sizeof(buf), "%s: %d\n", "Current length", length_time);
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s", buf);
       }
       else if(is_arg_contain(client_info, 1, "loop")) {
 	char buf[64];
@@ -2149,27 +2150,26 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 
 	switch(gGui->playlist.loop) {
 	case PLAYLIST_LOOP_NO_LOOP:
-	  sprintf(buf, "%s%s", buf, "'No Loop'");
+	  strlcat(buf, "'No Loop'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_LOOP:
-	  sprintf(buf, "%s%s", buf, "'Loop'");
+	  strlcat(buf, "'Loop'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_REPEAT:
-	  sprintf(buf, "%s%s", buf, "'Repeat'");
+	  strlcat(buf, "'Repeat'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_SHUFFLE:
-	  sprintf(buf, "%s%s", buf, "'Shuffle'");
+	  strlcat(buf, "'Shuffle'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_SHUF_PLUS:	
-	  sprintf(buf, "%s%s", buf, "'Shuffle forever'");
+	  strlcat(buf, "'Shuffle forever'", sizeof(buf));
 	  break;
 	default:
-	  sprintf(buf, "%s%s", buf, "'!!Unknown!!'");
+	  strlcat(buf, "'!!Unknown!!'", sizeof(buf));
 	  break;
 	}
 
-	sprintf(buf, "%s.\n", buf);
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s.\n", buf);
       }
     }
     else if(nargs >= 2) {
@@ -2435,11 +2435,11 @@ static void do_event(commands_t *cmd, client_info_t *client_info) {
 	while(events_struct[i].name && strncmp(arg, events_struct[i].name, strlen(arg))) i++;
       }
 
-      if((i >= 0) && (i < ((sizeof(events_struct) / sizeof(events_struct[0])) - 1))) {
+      if((i >= 0) && ((size_t)i < ((sizeof(events_struct) / sizeof(events_struct[0])) - 1))) {
 	xine_event.type = events_struct[i].type;
       }
       else {
-	int   value;
+	size_t value;
 	int   input_numbers[] = {
 	  XINE_EVENT_INPUT_NUMBER_0, XINE_EVENT_INPUT_NUMBER_1, XINE_EVENT_INPUT_NUMBER_2,
 	  XINE_EVENT_INPUT_NUMBER_3, XINE_EVENT_INPUT_NUMBER_4, XINE_EVENT_INPUT_NUMBER_5,
@@ -2448,10 +2448,10 @@ static void do_event(commands_t *cmd, client_info_t *client_info) {
 	};
 	  
 
-	if(((sscanf(arg, "%d", &value)) == 1) ||
-	   ((sscanf(arg, "+%d", &value)) == 1)) {
+	if(((sscanf(arg, "%zu", &value)) == 1) ||
+	   ((sscanf(arg, "+%zu", &value)) == 1)) {
 	  
-	  if((value >= 0) && (value < (sizeof(input_numbers) / sizeof(input_numbers[0]))))
+	  if(value < (sizeof(input_numbers) / sizeof(input_numbers[0])))
 	    xine_event.type = input_numbers[value];
 	}
       }
@@ -2552,7 +2552,7 @@ static void do_halt(commands_t *cmd, client_info_t *client_info) {
 static void network_messenger(void *data, char *message) {
   int socket = (int) data;
   
-  sock_write(socket, message);
+  sock_write(socket, "%s", message);
 }
 
 static void do_snap(commands_t *cmd, client_info_t *client_info) {
@@ -2577,7 +2577,7 @@ static void say_hello(client_info_t *client_info) {
   else {
     snprintf(buf, sizeof(buf), "%s %s %s\n", PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
-  sock_write(client_info->socket, buf);
+  sock_write(client_info->socket,"%s",  buf);
   
 }
 
@@ -2637,11 +2637,11 @@ static void parse_destock_remain(client_info_t *client_info) {
 	p++;
       
       if(p)
-	snprintf(remaining, sizeof(remaining), "%s", (_atoa(p)));
+	strcpy(remaining, (_atoa(p)));
       
       client_info->command.line = (char *) realloc(client_info->command.line, sizeof(char *) * (strlen(c) + 1));
       
-      sprintf(client_info->command.line, "%s", c);
+      strcpy(client_info->command.line, c);
       
       if(p) {
 	/* remove last ';' */
@@ -2656,7 +2656,7 @@ static void parse_destock_remain(client_info_t *client_info) {
 	if(strlen(remaining)) {
 	  client_info->command.remain = (char *) realloc(client_info->command.remain, 
 							 sizeof(char *) * (strlen(remaining) + 1));
-	  sprintf(client_info->command.remain, "%s", remaining);
+	  strcpy(client_info->command.remain, remaining);
 	}
 	else {
 	  SAFE_FREE(client_info->command.remain);
@@ -2672,7 +2672,7 @@ static void parse_destock_remain(client_info_t *client_info) {
       client_info->command.line = (char *) realloc(client_info->command.line, 
 						   sizeof(char *) * (strlen(client_info->command.remain) + 1));
       
-      sprintf(client_info->command.line, "%s", client_info->command.remain);
+      strcpy(client_info->command.line, client_info->command.remain);
       
       SAFE_FREE(client_info->command.remain);
     }
@@ -2767,7 +2767,7 @@ static void parse_command(client_info_t *client_info) {
 	  if((*(pcmd - 1) != '\\') && (get_quote == 0)) {
 
 	  __end_args:
-	    sprintf(client_info->command.args[nargs], "%s", _atoa(buf));
+	    strcpy(client_info->command.args[nargs], _atoa(buf));
 	    nargs++;
 	    memset(&buf, 0, sizeof(buf));
 	    pb = buf;
@@ -2793,8 +2793,7 @@ static void parse_command(client_info_t *client_info) {
   else {
     client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
     
-    memset(client_info->command.command, 0, sizeof(client_info->command.command));
-    sprintf(client_info->command.command, "%s", client_info->command.line);
+    strcpy(client_info->command.command, client_info->command.line);
   }
 
 #if 0
@@ -2981,15 +2980,15 @@ static void *server_thread(void *data) {
 
   while(gGui->running) {
     client_info_t       *client_info;
-    int                  lsin;
+    socklen_t            lsin;
     pthread_t            thread_client;
     
     msock = sock_serv(service, "tcp", 5);
     
     client_info = (client_info_t *) xine_xmalloc(sizeof(client_info_t));
-    lsin = sizeof(client_info->sin);
+    lsin = sizeof(client_info->fsin.in);
     
-    client_info->socket = accept(msock, (struct sockaddr *)&(client_info->sin), &lsin);
+    client_info->socket = accept(msock, &(client_info->fsin.sa), &lsin);
     client_info->authentified = is_client_authorized(client_info);
     
     if(client_info->socket < 0) {
