@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.c,v 1.71 2004/07/23 13:12:40 f1rmb Exp $
+ * $Id: label.c,v 1.72 2006/05/22 17:53:26 mshopf Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -52,7 +52,7 @@ static void _create_label_pixmap(xitk_widget_t *w) {
     if((strlen(private_data->label)) > private_data->length)
       sprintf(_label, "%s *** %s", private_data->label, private_data->label);
     else
-      sprintf(_label, "%s", private_data->label);
+      strcpy(_label, private_data->label);
     
   }
   else
