@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menu.c,v 1.23 2004/04/26 21:53:34 f1rmb Exp $
+ * $Id: menu.c,v 1.31 2006/11/06 00:23:33 fsck-p Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -163,6 +163,25 @@ static int _menu_branch_have_branch(menu_node_t *branch) {
   }
   return 0;
 }
+static int _menu_branch_is_in_curbranch(menu_node_t *branch) {
+  menu_private_data_t  *private_data = (menu_private_data_t *) branch->widget->private_data;
+  menu_node_t          *me = private_data->curbranch;
+
+  if(!me)
+    return 0;
+
+  while(branch != me) {
+    if(!me->prev)
+      return 0;
+
+    while(me->prev) {
+      me = me->prev;
+      if(me->prev && me->prev->branch == me)
+	break;
+    }
+  }
+  return 1;
+}
 
 #ifdef DUMP_MENU
 #ifdef	__GNUC__
@@ -277,7 +296,7 @@ static int _menu_is_title_in_branch(menu_node_t *branch) {
 static menu_node_t *_menu_get_wider_menuitem_node(menu_node_t *branch) {
   menu_node_t  *me = branch;
   menu_node_t  *max = NULL;
-  int           len = 0;
+  size_t        len = 0;
   
   while(me) {
     if((!_menu_is_separator(me->menu_entry)) && 
@@ -306,7 +325,7 @@ static int _menu_branch_have_shortcut(menu_node_t *branch) {
 static menu_node_t *_menu_get_wider_shortcut_node(menu_node_t *branch) {
   menu_node_t  *me = branch;
   menu_node_t  *max = NULL;
-  int           len = 0;
+  size_t        len = 0;
   
   while(me) {
     if((!_menu_is_separator(me->menu_entry)) && 
@@ -322,6 +341,29 @@ static menu_node_t *_menu_get_wider_shortcut_node(menu_node_t *branch) {
   return max;
 }
 
+static menu_node_t *_menu_get_branch_from_node(menu_node_t *node) {
+  menu_node_t          *me = node;
+  menu_private_data_t  *private_data = (menu_private_data_t *) me->widget->private_data;
+
+  while(me->prev) {
+    if(me->prev->branch == me)
+      return me;
+    me = me->prev;
+  }
+
+  return private_data->mtree->first;
+}
+
+static menu_node_t *_menu_get_prev_branch_from_node(menu_node_t *node) {
+  menu_node_t          *me;
+
+  me = _menu_get_branch_from_node(node);
+  if(me->prev)
+    return _menu_get_branch_from_node(me->prev);
+  else
+    return NULL;
+}
+
 static void _menu_destroy_menu_window(menu_window_t **mw) {
 
   xitk_unregister_event_handler(&(*mw)->key);
@@ -342,12 +384,46 @@ static void _menu_destroy_subs(menu_private_data_t *private_data, menu_window_t
   menu_window_t *mw;
   
   mw = (menu_window_t *) xitk_list_last_content(private_data->menu_windows);
+  if(!mw || (mw == menu_window))
+    /* Nothing to be done, save useless following efforts */
+    return;
   while(mw && (mw != menu_window)) {
     xitk_list_delete_current(private_data->menu_windows);
     _menu_destroy_menu_window(&mw);
     mw = (menu_window_t *) xitk_list_last_content(private_data->menu_windows);
   }
+  /* Set focus to parent menu */
+  if(xitk_is_window_visible(menu_window->display, (xitk_window_get_window(menu_window->xwin)))) {
+    int    retry = 0;
+    Window focused_win;
+
+    do {
+      int revert;
+
+      XLOCK(menu_window->display);
+      XSetInputFocus(menu_window->display, (xitk_window_get_window(menu_window->xwin)),
+		     RevertToParent, CurrentTime);
+      XSync(menu_window->display, False);
+      XUNLOCK(menu_window->display);
+
+      /* Retry until the WM was mercyful to give us the focus (but not indefinitely) */
+      xitk_usec_sleep(5000);
+      XLOCK(menu_window->display);
+      XGetInputFocus(menu_window->display, &focused_win, &revert);
+      XUNLOCK(menu_window->display);
+
+    } while((focused_win != xitk_window_get_window(menu_window->xwin)) && (retry++ < 30));
+  }
 }
+
+static int _menu_show_subs(menu_private_data_t *private_data, menu_window_t *menu_window) {
+  menu_window_t *mw;
+
+  mw = (menu_window_t *) xitk_list_last_content(private_data->menu_windows);
+
+  return (mw != menu_window);
+}
+
 static void _menu_hide_menu(menu_private_data_t *private_data) {
   menu_window_t *mw;
   
@@ -429,61 +505,39 @@ static void _menu_click_cb(xitk_widget_t *w, void *data) {
   if(_menu_is_branch(me->menu_entry)) {
     
     if(me->branch) {
-      menu_node_t   *mebr = me;
-      
-      while(mebr->branch) {
+
+      if(!_menu_branch_is_in_curbranch(me->branch)) {
+	int   wx = 0, wy = 0, x = 0, y = 0;
 	
-	if(private_data->curbranch && (mebr->branch != private_data->curbranch)) {
-	  int   wx = 0, wy = 0, x = 0, y = 0;
+	xitk_window_get_window_position(private_data->imlibdata, 
+					me->menu_window->xwin, &wx, &wy, NULL, NULL);
+	xitk_get_widget_pos(me->button, &x, &y);
+	
+	x += (xitk_get_widget_width(me->button)) + wx;
+	y += wy;
+	
+	_menu_destroy_subs(private_data, me->menu_window);
 	  
-	  xitk_window_get_window_position(private_data->imlibdata, 
-					  mebr->menu_window->xwin, &wx, &wy, NULL, NULL);
-	  xitk_get_widget_pos(mebr->button, &x, &y);
-	  
-	  x += (xitk_get_widget_width(mebr->button)) + wx;
-	  x -= 10;
-	  y += wy;
-	  
-	  
-	  if(me->branch && (me->branch == mebr->branch))
-	    _menu_destroy_subs(private_data, mebr->menu_window);
-	    
-	  _menu_create_menu_from_branch(mebr->branch, mebr->widget, x, y);
-	}
-	else {
-	  
-	  if(private_data->curbranch) {
-	    XLOCK(private_data->curbranch->menu_window->display);
-	    XRaiseWindow(private_data->curbranch->menu_window->display, 
-			 xitk_window_get_window(private_data->curbranch->menu_window->xwin));
-	    XSetInputFocus(private_data->curbranch->menu_window->display, 
-			   (xitk_window_get_window(private_data->curbranch->menu_window->xwin)),
-			   RevertToParent, CurrentTime);
-	    XUNLOCK(private_data->curbranch->menu_window->display);
-	  }
-	}
-
-	mebr = mebr->branch;
+	_menu_create_menu_from_branch(me->branch, me->widget, x, y);
+      }
+      else {
+	XLOCK(me->branch->menu_window->display);
+	XRaiseWindow(me->branch->menu_window->display, 
+		     xitk_window_get_window(me->branch->menu_window->xwin));
+	XSetInputFocus(me->branch->menu_window->display, 
+		       (xitk_window_get_window(me->branch->menu_window->xwin)),
+		       RevertToParent, CurrentTime);
+	XUNLOCK(me->branch->menu_window->display);
       }
-
-      private_data->curbranch = me->branch;
-
-      return;
     }
     else {
-      menu_node_t *mnode = (private_data->curbranch && private_data->curbranch->prev) ? 
-	private_data->curbranch->prev : private_data->mtree->first;
-      
-      _menu_destroy_subs(private_data, me->menu_window);
-      private_data->curbranch = mnode;
+      if(_menu_show_subs(private_data, me->menu_window)) {
+	_menu_destroy_subs(private_data, me->menu_window);
+	private_data->curbranch = _menu_get_branch_from_node(me);
+      }
     }
-
   }
-#ifdef DEBUG_MENU
-  else if(_menu_is_separator(me->menu_entry))
-    printf("SEPARATOR\n");
-#endif
-  else if(_menu_is_check(me->menu_entry)) {
+  else if(!_menu_is_title(me->menu_entry) && !_menu_is_separator(me->menu_entry)) {
     _menu_hide_menu(private_data);
 
     if(me->menu_entry->cb)
@@ -491,16 +545,10 @@ static void _menu_click_cb(xitk_widget_t *w, void *data) {
     
     xitk_menu_destroy(widget);
   }
-  else {
-    if(!_menu_is_title(me->menu_entry)) {
-      _menu_hide_menu(private_data);
-
-      if(me->menu_entry->cb)
-	me->menu_entry->cb(widget, me->menu_entry, me->menu_entry->user_data);
-      
-      xitk_menu_destroy(widget);
-    }
-  }
+#ifdef DEBUG_MENU
+  if(_menu_is_separator(me->menu_entry))
+    printf("SEPARATOR\n");
+#endif
 }
 
 static void _menu_handle_xevents(XEvent *event, void *data) {
@@ -724,14 +772,40 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
 			  (DefaultScreen(private_data->imlibdata->x.disp)));
   XUNLOCK(private_data->imlibdata->x.disp);
   
-  if((x + (wwidth + 2)) > swidth)
-    x = swidth - (wwidth + 2);
-  if((y + (wheight + 2)) > sheight)
-    y = sheight - (wheight + 2);
-  
+  if(branch != private_data->mtree->first) {
+    x -= 4; /* Overlap parent menu but leave text and symbols visible */
+    y -= 1; /* Top item of submenu in line with parent item */
+  }
+  else {
+    x++; y++; /* Upper left corner 1 pix distance to mouse pointer */
+  }
+
+  /* Check if menu fits on screen and adjust position if necessary in a way */
+  /* that it doesn't obscure the parent menu or get under the mouse pointer */
+
+  if((x + (wwidth + 2)) > swidth) { /* Exceeds right edge of screen */
+    if(branch != private_data->mtree->first) {
+      /* Align right edge of submenu to left edge of parent item */
+      x -= xitk_get_widget_width(branch->prev->button) + (wwidth + 2) - 4;
+    }
+    else {
+      /* Align right edge of top level menu to right edge of screen */
+      x  = swidth - (wwidth + 2);
+    }
+  }
+  if((y + (wheight + 2)) > sheight) { /* Exceeds bottom edge of screen */
+    if(branch != private_data->mtree->first) {
+      /* Align bottom edge of submenu for bottom item in line with parent item */
+      y -= wheight - xitk_get_widget_height(branch->prev->button);
+    }
+    else {
+      /* Align bottom edge of top level menu to requested (i.e. pointer) pos */
+      y -= (wheight + 2) + 1;
+    }
+  }
+
   xwin = xitk_window_create_simple_window(private_data->imlibdata, 
 					  x, y, wwidth + 2, wheight + 2);
-  xitk_set_wm_window_type(xitk_window_get_window(xwin), WINDOW_TYPE_DOCK);
 
   if(bsep || btitle) {
     int width, height;
@@ -856,11 +930,16 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
   xitk_set_layer_above((xitk_window_get_window(xwin)));
   
   XLOCK(private_data->imlibdata->x.disp);
-  XSetTransientForHint(private_data->imlibdata->x.disp, 
-		       (xitk_window_get_window(xwin)), private_data->parent_wlist->win);
+  /* Set transient-for-hint to the immediate predecessor,     */
+  /* so window stacking of submenus is kept upon raise/lower. */
+  if(branch == private_data->mtree->first)
+    XSetTransientForHint(private_data->imlibdata->x.disp,
+			 (xitk_window_get_window(xwin)), private_data->parent_wlist->win);
+  else
+    XSetTransientForHint(private_data->imlibdata->x.disp,
+			 (xitk_window_get_window(xwin)), (xitk_window_get_window(branch->prev->menu_window->xwin)));
   XUNLOCK(private_data->imlibdata->x.disp);
 
-
   menu_window->key = xitk_register_event_handler("xitk menu",
 						 (xitk_window_get_window(menu_window->xwin)), 
 						 _menu_handle_xevents,
@@ -876,6 +955,18 @@ static void _menu_create_menu_from_branch(menu_node_t *branch, xitk_widget_t *w,
   
   while(!xitk_is_window_visible(private_data->imlibdata->x.disp, (xitk_window_get_window(xwin))))
     xitk_usec_sleep(5000);
+
+  if(!(xitk_get_wm_type() & WM_TYPE_KWIN))
+    /* WINDOW_TYPE_MENU seems to be the natural choice. */
+    xitk_set_wm_window_type(xitk_window_get_window(xwin), WINDOW_TYPE_MENU);
+  else
+    /* However, KWin has unacceptable behaviour for WINDOW_TYPE_MENU in  */
+    /* our transient-for scheme: The transient-for window must be mapped */
+    /* and the transient-for window or another transient window (incl.   */
+    /* the menu itself) must have focus, otherwise it unmaps the menu.   */
+    /* This causes menus not to be shown under many several conditions.  */
+    /* WINDOW_TYPE_DOCK is definitely the right choice for KWin.         */
+    xitk_set_wm_window_type(xitk_window_get_window(xwin), WINDOW_TYPE_DOCK);
   
   XLOCK(private_data->imlibdata->x.disp);
   XSetInputFocus(private_data->imlibdata->x.disp, 
@@ -914,6 +1005,25 @@ int xitk_menu_show_sub_branchs(xitk_widget_t *w) {
   return ret;
 }
 
+void xitk_menu_destroy_branch(xitk_widget_t *w) {
+
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MENU) && 
+	   (w->type & WIDGET_TYPE_LABELBUTTON))) {
+    menu_node_t         *me = labelbutton_get_user_data(w);    
+
+    me = _menu_get_prev_branch_from_node(me);
+    if(me) {
+      menu_private_data_t *private_data = (menu_private_data_t *) me->widget->private_data;
+
+      _menu_destroy_subs(private_data, me->menu_window);
+      private_data->curbranch = me;
+    }
+    else {
+      xitk_menu_destroy(xitk_menu_get_menu(w));
+    }
+  }
+}
+
 void menu_auto_pop(xitk_widget_t *w) {
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MENU) && 
@@ -929,14 +1039,9 @@ void menu_auto_pop(xitk_widget_t *w) {
     if(_menu_is_branch(me->menu_entry))
       xitk_labelbutton_callback_exec(w);
     else {
-      if(xitk_menu_show_sub_branchs(widget)) {
-	menu_node_t *mnode;
-	
-	mnode = (private_data->curbranch && private_data->curbranch->prev) ? 
-	  private_data->curbranch->prev : private_data->mtree->first;
-	
+      if(_menu_show_subs(private_data, me->menu_window)) {
 	_menu_destroy_subs(private_data, me->menu_window);
-	private_data->curbranch = mnode;
+	private_data->curbranch = _menu_get_branch_from_node(me);
       }
     }
   }
@@ -969,6 +1074,7 @@ xitk_widget_t *xitk_noskin_menu_create(xitk_widget_list_t *wl,
   private_data->menu_windows = xitk_list_new();
   private_data->x            = x;
   private_data->y            = y;
+  private_data->curbranch    = NULL;
 
 
   if(!m->menu_tree) {
